import java.util.Vector;

/** BaseConocimientos.java
   Implementa la base de conocimiento de un sistema experto que
   juega al conecta 4, es decir, el mecanismo de obtencin de los
   posibles movimientos dada una base de hechos.
   @author Alberto Alonso Ruibal
*/

class BaseConocimientos {

	BaseHechos oBaseHechos;

	/**
	   Dada una base de hechos construye su base de conocimientos
	*/
	BaseConocimientos(BaseHechos oBaseHechosOrigen) {
		oBaseHechos = oBaseHechosOrigen;
	}

	/**
	   Devuelve un vector con las operciones posibles (los movimientos
	   posibles). Es matar moscar a caonazos, pero vale para sistemas
	   expertos ms grandes.
	*/
	Vector listaOperaciones() {
		int i;
		Vector<Operacion> oVector = new Vector<Operacion>();
		Operacion oOperacion;

		for (i = 0; i < 7; i++) {
			oOperacion = new Operacion(i);
			if (operacionValida(oOperacion))
				oVector.addElement(oOperacion);
		}

		return oVector;
	}

	/**
	   Devuelve true si un movimiento es valido
	 */
	boolean operacionValida(Operacion oOperacion) {
		return ((oOperacion.intValue() >= 0) &&
				(oOperacion.intValue() <= 6) &&
				(hay4enRaya() == 0) &&
				(oBaseHechos.iTablero[oOperacion.intValue()][5] == 0));
	}

	/**
	   Devuelve 1 si hay un 4 en raya de las "blancas", -1 si
	   hay un 4 en raya de las "negras" y 0 si no hay ningn 4
	   en raya.
	 */
	int hay4enRaya() {
		int i, j, k, iAux, iAux2, iCont, iCont2;
		// Horizontales
		for (j = 0; j < 6; j++) {
			iAux = 0; iCont = 0;
			for (i = 0; i < 7; i++) {
				if (oBaseHechos.iTablero[i][j] == iAux) iCont += iAux;
				else {
					iCont = 0;
					iAux = oBaseHechos.iTablero[i][j];
				}
				if (iCont == 3) return 1;
				if (iCont == -3) return -1;
			}
		}
		// Verticales
		for (i = 0; i < 7; i++) {
			iAux = 0; iCont = 0;
			for (j = 0; j < 6; j++) {
				if (oBaseHechos.iTablero[i][j] == iAux) iCont += iAux;
				else {
					iCont = 0;
					iAux = oBaseHechos.iTablero[i][j];
				}
				if (iCont == 3) return 1;
				if (iCont == -3) return -1;
			}
		}
		// Diagonales
		for (k = 0; k < 7; k++) {
			iAux = 0; iCont = 0;
			iAux2 = 0; iCont2 = 0;
			if (k <= 2) i = 0; else i = k - 2;
			if (k >= 2) j = 0; else j = 2 - k;
			for (; (j < 6) && (i < 7) ; i++,j++) {
				if (oBaseHechos.iTablero[i][j] == iAux) iCont += iAux;
				else {
					iCont = 0;
					iAux = oBaseHechos.iTablero[i][j];
				}
				if (oBaseHechos.iTablero[i][5 - j] == iAux2) iCont2 += iAux2;
				else {
					iCont2 = 0;
					iAux2 = oBaseHechos.iTablero[i][5 - j];
				}
				if ((iCont == 3) || (iCont2 == 3)) return 1;
				if ((iCont == -3) || (iCont2 == -3)) return -1;
			}
		}
		return 0;
	}

}







